package Class::SingletonMethod;

use 5.006002; # encourage upgrade
use strict;
use warnings;
use Carp;

our $VERSION = sprintf "%d.%02d", q$Revision: 2.0 $ =~ /(\d+)/g;

croak << "EOT";
Please use either one of:

  Universal::SingletonMethod #  what used to be Class::SingletonMethod 1.0
  Class::SingletonMethod::Std
  Class::SingletonMethod::Hash

Instead of Class::SingletonMethod now.
EOT

1; # never reached.

__END__

=head1 NAME

Class::SingletonMethod - a placeholder for singleton method implementors

=head1 SYNOPSIS

    use Universal::SingletonMethod; # what used to be Class::SingletonMethod;
    use base 'Class::SingletonMethod::Std';  # new way
    use base 'Class::SingletonMethod::Hash'; # yet another new way
    my $b = Some::Class->new; 

=head1 DESCRIPTION

Simon Conzens first came up with how to implement singleton methods in
perl.  It is discussed in Advanced Perl Programming 2nd. ed.

Though Extremely powerful, some people don't like the idea of doing
that by "singleton class".  Dan Kogai thought about it and came up
with another ideas that are L<Class::SingletonMethod::Std> and
L<Class::SingletonMethod::Hash>.

With the avent of more ways to do it, the module hierarchy is reorganized.

=over 2

=item *

The original Class::SingletonMethod is now L<Universal::SingletonMethod>.

=item *

This module itself croaks with a message notifying the change.

=item *

L<Class::SingletonMethod::Std> and L<Class::SingletonMethod::Hash>
are added.

=back

=head2 What is a singleton method?

The Ruby FAQ defines singleton methods like so:

    (Q)     What is a singleton method?

    (A)     A singleton method is defined for the particular object but
            in the class. A singleton method allows appending or
            changing methods without making subclasses.

            msg = "Hello"
            def msg.print
              $>.print self, "\n"
            end
            msg.print   #=> Hello

That is, you can add or override methods on a per-object basis. 

If you know Javascript which implements objects on prototypes rather than classes, the idea of singleton methods does not sound so alien.

  var msg = {'string':'Hello'};
  msg.print = function(){
    document.write(this.string);
  };
  msg.print();

Also note on Perl6, singleton methods are natively supported.

  my $msg = "Hello";
  $msg.meta.add_method('world' 
                       => method ($self:){ say $self, " world!" });
  $msg.world;

=head1 AUTHOR

Simon Cozens, C<simon@cpan.org>

Dan Kogai, C<dankogai@dan.co.jp>

=head1 SEE ALSO

ruby L<http://www.ruby-lang.org/>

Advanced Perl Programming, 2nd ed.
L<http://www.oreilly.com/catalog/advperl2/>

=cut
